// Copyright Peter Širka, Web Site Design s.r.o. (www.petersirka.sk)
//
// Permission is hereby granted, free of charge, to any person obtaining a
// copy of this software and associated documentation files (the
// "Software"), to deal in the Software without restriction, including
// without limitation the rights to use, copy, modify, merge, publish,
// distribute, sublicense, and/or sell copies of the Software, and to permit
// persons to whom the Software is furnished to do so, subject to the
// following conditions:
//
// The above copyright notice and this permission notice shall be included
// in all copies or substantial portions of the Software.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
// MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN
// NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM,
// DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
// OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE
// USE OR OTHER DEALINGS IN THE SOFTWARE.

'use strict';

var urlParser = require('url');
var http = require('http');
var https = require('https');
var util = require('util');
var path = require('path');
var fs = require('fs');
var events = require('events');
var crypto = require('crypto');
var regexpMail = new RegExp('^[a-zA-Z0-9-_.]+@[a-zA-Z0-9.-]+\\.[a-zA-Z]{2,4}$');
var regexpUrl = new RegExp('^(http[s]?:\\/\\/(www\\.)?|ftp:\\/\\/(www\\.)?|www\\.){1}([0-9A-Za-z-\\.@:%_\+~#=]+)+((\\.[a-zA-Z]{2,3})+)(/(.)*)?(\\?(.)*)?');
var expressionCache = {};

var ENCODING = 'utf8';
var UNDEFINED = 'undefined';
var STRING = 'string';
var FUNCTION = 'function';
var NUMBER = 'number';
var OBJECT = 'object';
var BOOLEAN = 'boolean';

if (typeof(setImmediate) === UNDEFINED) {
	global.setImmediate = function(cb) {
		process.nextTick(cb);
	};
}

var hasOwnProperty = Object.prototype.hasOwnProperty;

/*
	Expression declaration
	@query {String}
	@params {String Array}
	@values {Object Array}
	return {Function}
*/
function expression(query, params) {

	var name = params.join(',');
	var fn = expressionCache[query + '-' + name];

	if (!fn) {
		fn = eval('(function(' + name +'){' + (query.indexOf('return') === -1 ? 'return ' : '') + query + '})');
		expressionCache[query + name] = fn;
	}

	var values = [];

	for (var i = 2; i < arguments.length; i++)
		values.push(arguments[i]);

	return (function() {
		var arr = [];

		for (var i = 0; i < arguments.length; i++)
			arr.push(arguments[i]);

		for (var i = 0; i < values.length; i++)
			arr.push(values[i]);

		return fn.apply(this, arr);
	});
}

global.expression = expression;


/*
	@obj {Object}
*/
exports.isEmpty = function(obj) {

	if (obj === null)
		return true;

	if (obj.length && obj.length > 0)
		return false;

	if (obj.length === 0)
		return true;

	for (var key in obj) {
		if (hasOwnProperty.call(obj, key))
		return false;
	}
	return true;
};


/*
	Send request to URL
	@url {String}
	@method {String}
    @data {String}
    @callback {Function} :: function(error, data, statusCode, headers)
    @headers {Object} :: optional, default {}
    @encoding {String} :: optional, default utf8
    @timeout {Number} :: optional, default 10000
*/
exports.request = function(url, method, data, callback, headers, encoding, timeout) {

	var uri = urlParser.parse(url);
	var h = {};
	var isJSON = typeof(data) === OBJECT;

	encoding = encoding || ENCODING;
	method = (method || '').toString().toUpperCase();

	if (method !== 'GET')
		h['Content-Type'] = 'application/x-www-form-urlencoded';

	if (isJSON)
		h['Content-Type'] = 'application/json';

	util._extend(h, headers);

	var options = { protocol: uri.protocol, auth: uri.auth, method: method, hostname: uri.hostname, port: uri.port, path: uri.path, agent: false, headers: h };

	var response = function(res) {

		if (!callback) {
			res.resume();
			return;
		}

		res._buffer = '';

		res.on('data', function(chunk) {
			this._buffer += chunk.toString(encoding);
		});

		res.on('end', function() {
			callback(null, this._buffer, res.statusCode, res.headers);
		});

		res.resume();
	};

	var con = options.protocol === 'https:' ? https : http;

	try
	{

		var isPOST = method === 'POST' || method === 'PUT';
		var req = isPOST ? callback ? con.request(options, response) : con.request(options) : callback ? con.get(options, response) : con.get(options);

		if (callback) {
			req.on('error', function(error) {
				callback(error, null, 0, {});
			});

			req.setTimeout(timeout || 10000, function() {
				callback(new Error(exports.httpStatus(408)), null, 0, {});
			});
		}

		if (isPOST)
			req.end(isJSON ? JSON.stringify(data) : (data || '').toString(), encoding);

	} catch (ex) {
		if (callback)
			callback(ex, null, 0, {});
		return false;
	}

	return true;
};

/*
	TRIM string properties
	@obj {Object}
	return {Object}
*/
exports.trim = function(obj) {

	var type = typeof(obj);

	if (type === STRING)
		return obj.trim();

	if (type !== OBJECT)
		return obj;

	Object.keys(obj).forEach(function(name) {
		var val = obj[name];

		if (typeof(val) === OBJECT) {
			exports.trim(val);
			return;
		}

		if (typeof(val) !== STRING)
			return;

		obj[name] = val.trim();
	});

	return obj;
};

/*
	Empty function
*/
exports.noop = function() {};

/*
	Get HTTP Status description
	@code {Number}
	@addCode {Boolean} :: optional, default true
	return {String}
*/
exports.httpStatus = function(code, addCode) {

	var plus = addCode || true ? code + ': ' : '';
	switch(code) {
		case 200: return plus + 'OK';
		case 201: return plus + 'Created';
		case 202: return plus + 'Accepted';
		case 203: return plus + 'Non-Authoritative Information';
		case 204: return plus + 'No Content';
		case 205: return plus + 'Reset Content';
		case 206: return plus + 'Partial Content';
		case 300: return plus + 'Multiple Choices';
		case 301: return plus + 'Moved Permanently';
		case 302: return plus + 'Found';
		case 303: return plus + 'See Other';
		case 304: return plus + 'Not Modified';
		case 306: return plus + 'Switch Proxy';
		case 307: return plus + 'Temporary Redirect';
		case 308: return plus + 'Resume Incomplete';
		case 400: return plus + 'Bad Request';
		case 401: return plus + 'Unauthorized';
		case 402: return plus + 'Payment Required';
		case 403: return plus + 'Forbidden';
		case 404: return plus + 'Not Found';
		case 405: return plus + 'Method Not Allowed';
		case 406: return plus + 'Not Acceptable';
		case 407: return plus + 'Proxy Authentication Required';
		case 408: return plus + 'Request Timeout';
		case 409: return plus + 'Conflict';
		case 410: return plus + 'Gone';
		case 411: return plus + 'Length Required';
		case 412: return plus + 'Precondition Failed';
		case 413: return plus + 'Request Entity Too Large';
		case 414: return plus + 'Request-URI Too Long';
		case 415: return plus + 'Unsupported Media Type';
		case 416: return plus + 'Requested Range Not Satisfiable';
		case 417: return plus + 'Expectation Failed';
		case 431: return plus + 'Request Header Fields Too Large';
		case 500: return plus + 'Internal Server Error';
		case 501: return plus + 'Not Implemented';
		case 502: return plus + 'Bad Gateway';
		case 503: return plus + 'Service Unavailable';
		case 504: return plus + 'Gateway Timeout';
		case 505: return plus + 'HTTP Version Not Supported';
		case 511: return plus + 'Network Authentication Required';
	}
	return null;
};

/*
	Extend object
	@target {Object}
	@source {Object}
	@rewrite {Boolean} :: option, default false
	return {Object}
*/
exports.extend = function(target, source, rewrite) {

	if (target === null || source === null)
		return target;

	if (typeof(target) !== OBJECT || typeof(source) !== OBJECT)
		return target;

	var keys = Object.keys(source);
	var i = keys.length;

	while (i--) {

		var key = keys[i];

		if (rewrite || typeof(target[key]) === UNDEFINED)
			target[key] = source[key];
	}

	return target;
};

/*
	Copy values
	@target {Object}
	@source {Object}
	@rewrite {Boolean} :: option, default true
	return {Object}
*/
exports.copy = function(target, source, rewrite) {

	if (target === null || source === null)
		return target;

	if (typeof(target) !== OBJECT || typeof(source) !== OBJECT)
		return target;

	if (typeof(rewrite) === UNDEFINED)
		rewrite = true;

	var keys = Object.keys(source);
	var i = keys.length;

	while (i--) {

		var key = keys[i];

		if (typeof(target[key]) === UNDEFINED)
			continue;

		if (rewrite) {
			target[key] = source[key];
			continue;
		}

		target[key] = source[key];
	}

	return target;
};

/*
	Reduce object properties
	@source {Object}
	@prop {String array or Object} :: property name
	return @source
*/
exports.reduce = function(source, prop) {

	if (source === null || prop === null)
		return source;

	var type = typeof(prop);

	if (prop instanceof Array) {
		Object.keys(source).forEach(function(o) {
			if (prop.indexOf(o) === -1)
				delete source[o];
		});
	}

	if (type === OBJECT) {
		var obj = Object.keys(prop);
		Object.keys(source).forEach(function(o) {
			if (obj.indexOf(o) === -1)
				delete source[o];
		});
	}

	return source;
};

/*
	Is relative URL?
	@url {String}
	return {Boolean}
*/
exports.isRelative = function(url) {
	return !(url.substring(0, 2) === '//' || url.indexOf('http://') !== -1 || url.indexOf('https://') !== -1);
};

/*
	Encode HTML
	@str {String}
	return {String}
*/
exports.htmlEncode = function(str) {

	var type = typeof(str);

	if (type === UNDEFINED)
		return '';

	if (type !== STRING)
		str = str.toString();

	return str.htmlEncode();
};

/*
	Decode HTML
	@str {String}
	return {String}
*/
exports.htmlDecode = function(str) {

	var type = typeof(str);

	if (type === UNDEFINED)
		return '';

	if (type !== STRING)
		str = str.toString();

	return str.htmlDecode();
};

/*
	Is static file?
	@url {String}
	return {Boolean}
*/
exports.isStaticFile = function(url) {
	var pattern = /\.\w{2,8}($|\?)+/g;
	return pattern.test(url);
};

/*
	@str {String}
	return {Boolean}
*/
exports.isNullOrEmpty = function(str) {

	if (typeof(str) !== STRING)
		return true;

	return str.length === 0;
};

/*
	parseInt
	@obj {Object}
	@def {Number}
	return {Number}
*/
exports.parseInt = function(obj, def) {
	var type = typeof(obj);

	if (type === UNDEFINED)
		return def || 0;

	var str = type !== STRING ? obj.toString() : obj;
    return str.parseInt(def, 10);
};

/*
	parseFloat
	@obj {Object}
	@def {Number}
	return {Number}
*/
exports.parseFloat = function(obj, def) {
	var type = typeof(obj);

	if (type === UNDEFINED)
		return def || 0;

	var str = type !== STRING ? obj.toString() : obj;
    return str.parseFloat(def);
};

/*
	Is array?
	@obj {Object}
	return {Boolean}
*/
exports.isArray = function(obj) {
	return util.isArray(obj);
};

/*
	Is date?
	@obj {Object}
	return {Boolean}
*/
exports.isDate = function(obj) {
	return util.isDate(obj);
};

/*
	Get Content Type from extension
	@ext {String}
	return {String}
*/
exports.getContentType = function(ext) {

	if (ext[0] === '.')
		ext = ext.substring(1);

	var extension = {
		'ai': 'application/postscript',
		'appcache': 'text/cache-manifest',
		'avi': 'video/avi',
		'bin': 'application/octet-stream',
		'bmp': 'image/bmp',
		'css': 'text/css',
		'csv': 'text/csv',
		'doc': 'application/msword',
		'docx': 'application/msword',
		'dtd': 'application/xml-dtd',
		'eps': 'application/postscript',
		'exe': 'application/octet-stream',
		'gif': 'image/gif',
		'gzip': 'application/x-gzip',
		'htm': 'text/html',
		'html': 'text/html',
		'ico': 'image/x-icon',
		'ics': 'text/calendar',
		'ifb': 'text/calendar',
		'jpe': 'image/jpeg',
		'jpeg': 'image/jpeg',
		'jpg': 'image/jpeg',
		'js': 'text/javascript',
		'm4a': 'audio/mp4a-latm',
		'm4v': 'video/x-m4v',
		'md': 'text/markdown',
		'mid': 'audio/midi',
		'midi': 'audio/midi',
		'mov': 'video/quicktime',
		'mp3': 'audio/mpeg',
		'mp4': 'video/mp4',
		'mpe': 'video/mpeg',
		'mpeg': 'video/mpeg',
		'mpg': 'video/mpeg',
		'mpga': 'audio/mpeg',
		'mv4': 'video/mv4',
		'ogg': 'application/ogg',
		'pdf': 'application/pdf',
		'png': 'image/png',
		'ppt': 'application/vnd.ms-powerpoint',
		'pptx': 'application/vnd.ms-powerpoint',
		'ps': 'application/postscript',
		'rar': 'application/x-rar-compressed',
		'rtf': 'text/rtf',
		'sh': 'application/x-sh',
		'svg': 'image/svg+xml',
		'swf': 'application/x-shockwave-flash',
		'tar': 'application/x-tar',
		'tif': 'image/tiff',
		'tiff': 'image/tiff',
		'txt': 'text/plain',
		'wav': 'audio/x-wav',
		'webp': 'image/webp',
		'woff': 'font/woff',
		'xht': 'application/xhtml+xml',
		'xhtml': 'application/xhtml+xml',
		'xls': 'application/vnd.ms-excel',
		'xlsx': 'application/vnd.ms-excel',
		'xml': 'application/xml',
		'xpm': 'image/x-xpixmap',
		'xsl': 'application/xml',
		'xslt': 'application/xslt+xml',
		'zip': 'application/zip'
	};

	return extension[ext.toLowerCase()] || 'application/octet-stream';
};

/*
	Create ETag from string
	@text {String} :: filename
	@version {String} :: optional
	return {String}
*/
exports.etag = function(text, version) {
	var sum = 0;
	for (var i = 0; i < text.length; i++)
		sum += text.charCodeAt(i);
	return sum.toString() + (version ? ':' + version : '');
};

/*
	Add @delimiter to end of @path
	@path {String} :: filename
	@delimiter {String} :: optional, default /
	return {String}
*/
exports.path = function(path, delimiter) {
	delimiter = delimiter || '/';
	if (path[path.length - 1] === delimiter)
		return path;
	return path + delimiter;
};

/*
	Get random number
	@max {Number}
	@min {Number}
	return {Number}
*/
exports.random = function(max, min) {
	max = (max || 100000);
	min = (min || 0);
	return Math.floor(Math.random() * (max - min + 1)) + min;
};

/*
	Create unique identifier
	@max {Number} :: optional, default 40
	return {String}
*/
exports.GUID = function(max) {

	max = max || 40;

    var rnd = function () {
        return Math.floor(Math.random() * 65536).toString(16);
    };

    var str = '';
    for (var i = 0; i < (max / 4) + 1; i++)
		str += rnd();

    return str.substring(0, max);
};

/*
	Validate
	@model {Object} :: object to validate
	@properties {String array}
	@prepare {Function} : return utils.isValid() OR {Boolean} :: true is valid
	@builder {ErrorBuilder}
	@resource {Function} :: function(key) return {String}
	return {ErrorBuilder}
*/
exports.validate = function(model, properties, prepare, builder, resource) {

	if (typeof(builder) === FUNCTION && typeof(resource) === UNDEFINED) {
		resource = builder;
		builder = null;
	}

	var error = builder;

	if (!(error instanceof builders.ErrorBuilder))
		error = new builders.ErrorBuilder(resource);

	if (typeof(properties) === STRING) {
		var schema = builders.validation(properties);
		if (schema.length === 0)
			properties = properties.replace(/\s/g, '').split(',');
		else
			properties = schema;
	}

	if (typeof(model) === UNDEFINED || model === null)
		model = {};

	if (typeof(prepare) !== FUNCTION)
		throw new Error('Validate hasn\'t any method to validate properties.\nDefine delegate: framework.onValidate ...');

	for (var i = 0; i < properties.length; i++) {

		var type = typeof(value);
		var name = properties[i].toString();
		var value = (type === FUNCTION ? model[name]() : model[name]) || '';

		if (type === OBJECT) {
			error.add(exports.validate(value, properties, prepare, error, builder, resource));
			continue;
		}

		var result = prepare(name, value);

		if (typeof(result) === UNDEFINED)
			continue;

		type = typeof(result);

		if (type === STRING) {
			error.add(name, result);
			continue;
		}

		if (type === BOOLEAN) {
			if (!result)
				error.add(name, '@');
			continue;
		}

		if (result.isValid === false)
			error.add(name, result.error);
	}

	return error;
};

/*
	Validation object
	@isValid {Boolean}
	@error {String} :: optional, default @
	return {Object}
*/
exports.isValid = function(valid, error) {
	return { isValid: valid, error: error || '@' };
};

/*
	Email address validation
	@str {String}
	return {Boolean}
*/
exports.isEmail = function(str) {
	return (str || '').toString().isEmail();
};

/*
	URL address validation
	@str {String}
	return {Boolean}
*/
exports.isURL = function(str) {
	return (str || '').toString().isURL();
};

/*
	Combine path
	@arguments {String array}
	return {String}
*/
exports.combine = function() {
	return '.' + path.join.apply(this, arguments);
};

/*
	@str {String}
	return {String}
*/
exports.removeDiacritics = function(str) {
    var dictionaryA = ['á', 'ä', 'č', 'ď', 'é', 'ě', 'ť', 'ž', 'ú', 'ů', 'ü', 'í', 'ï', 'ô', 'ó', 'ö', 'š', 'ľ', 'ĺ', 'ý', 'ÿ', 'č', 'ř'];
    var dictionaryB = ['a', 'a', 'c', 'd', 'e', 'e', 't', 'z', 'u', 'u', 'u', 'i', 'i', 'o', 'o', 'o', 's', 'l', 'l', 'y', 'y', 'c', 'r'];
    var buf = [];
    for (var i = 0; i < str.length; i++) {
        var c = str[i];
        var isUpper = false;

        var index = dictionaryA.indexOf(c);
        if (index === -1) {
            index = dictionaryA.indexOf(c.toLowerCase());
            isUpper = true;
        }

        if (index === -1) {
            buf.push(c);
            continue;
        }

        c = dictionaryB[index];
        if (isUpper)
            c = c.toUpperCase();

        buf.push(c);
    }
    return buf.join('');
};

// Author: Haribabu Pasupathy
// http://stackoverflow.com/users/1679439/haribabu-pasupathy
exports.encode_WS = function(data){

    var bytesFormatted = [];
    bytesFormatted[0] = 129;

    if (data.length <= 125) {
        bytesFormatted[1] = data.length;
    } else if (data.length >= 126 && data.length <= 65535) {
        bytesFormatted[1] = 126;
        bytesFormatted[2] = (data.length >> 8) & 255;
        bytesFormatted[3] = (data.length) & 255;
    } else {
        bytesFormatted[1] = 127;
        bytesFormatted[2] = (data.length >> 56) & 255;
        bytesFormatted[3] = (data.length >> 48) & 255;
        bytesFormatted[4] = (data.length >> 40) & 255;
        bytesFormatted[5] = (data.length >> 32) & 255;
        bytesFormatted[6] = (data.length >> 24) & 255;
        bytesFormatted[7] = (data.length >> 16) & 255;
        bytesFormatted[8] = (data.length >> 8) & 255;
        bytesFormatted[9] = (data.length) & 255;
    }

    for (var i = 0; i < data.length; i++)
        bytesFormatted.push(data.charCodeAt(i));

    return bytesFormatted;
};

// Author: Haribabu Pasupathy
// http://stackoverflow.com/users/1679439/haribabu-pasupathy
exports.decode_WS = function(data) {

    var datalength = data[1] & 127;
    var indexFirstMask = 2;

    if (datalength === 126)
        indexFirstMask = 4;
    else if (datalength === 127)
        indexFirstMask = 10;

    var masks = data.slice(indexFirstMask, indexFirstMask + 4);
    var i = indexFirstMask + 4;
    var index = 0;
    var output = '';

    while (i < data.length)
        output += String.fromCharCode(data[i++] ^ masks[index++ % 4]);

    return output;
};

/*
	@type {String}
	@value {Number}
	return {Date}
*/
Date.prototype.add = function(type, value) {
	var self = this;
	switch(type) {
		case 's':
		case 'ss':
		case 'second':
		case 'seconds':
			self.setSeconds(self.getSeconds() + value);
			return self;
		case 'm':
		case 'mm':
		case 'minute':
		case 'minutes':
			self.setMinutes(self.getMinutes() + value);
			return self;
		case 'h':
		case 'hh':
		case 'hour':
		case 'hours':
			self.setHours(self.getHours() + value);
			return self;
		case 'd':
		case 'dd':
		case 'day':
		case 'days':
			self.setDate(self.getDate() + value);
			return self;
		case 'M':
		case 'MM':
		case 'month':
		case 'months':
			self.setMonth(self.getMonth() + value);
			return self;
		case 'y':
		case 'yyyy':
		case 'year':
		case 'years':
			self.setFullYear(self.getFullYear() + value);
			return self;
	}
	return self;
};

/*
	Format date to string
	@format {String}
	return {String}
*/
Date.prototype.format = function(format) {
	var self = this;

	var h = self.getHours();
	var m = self.getMinutes().toString();
	var s = self.getSeconds().toString();
	var M = (self.getMonth() + 1).toString();
	var yyyy = self.getFullYear().toString();
	var d = self.getDate().toString();

	var a = 'AM';
	var H = h.toString();


	if (h >= 12) {
		h -= 12;
		a = 'PM';
	}

	if (h === 0)
		h = 12;

	h = h.toString();

	var hh = h.padLeft(2, '0');
	var HH = H.padLeft(2, '0');
	var mm = m.padLeft(2, '0');
	var ss = s.padLeft(2, '0');
	var MM = M.padLeft(2, '0');
	var dd = d.padLeft(2, '0');
	var yy = yyyy.substring(2);

	return format.replace(/yyyy/g, yyyy).replace(/yy/g, yy).replace(/MM/g, MM).replace(/M/g, M).replace(/dd/g, dd).replace(/d/g, d).replace(/HH/g, HH).replace(/H/g, H).replace(/hh/g, hh).replace(/h/g, h).replace(/mm/g, mm).replace(/m/g, m).replace(/ss/g, ss).replace(/s/g, ss).replace(/a/g, a);
};

if (!String.prototype.trim) {
	String.prototype.trim = function() {
		return this.replace(/^[\s]+|[\s]+$/g, '');
	};
}

/*
	Count text in string
	@text {String}
	return {Number}
*/
String.prototype.count = function(text) {
    var index = 0;
    var count = 0;
    do {

        index = this.indexOf(text, index + text.length);

        if (index > 0)
            count++;

    } while (index > 0);
    return count;
};

String.prototype.parseDate = function() {
	return new Date(this.toString());
};

/*
	Contain string a array values?
	@value {String or String Array}
	@mustAll {Boolean} :: optional (default false), String must contains all items in String array
	return {Boolean}
*/
String.prototype.contains = function(value, mustAll) {

	var str = this.toString();

	if (typeof(value) === STRING)
		return str.indexOf(value) !== -1;

	for (var i = 0; i < value.length; i++) {
		var exists = str.indexOf(value[i]) !== -1;

		if (mustAll) {
			if (!exists)
				return false;
		} else if (exists)
			return true;
	}

	return mustAll ? true : false;
};

/*
	Parse configuration from string
	return {Object}
*/
String.prototype.configuration = function() {

	var arr = this.split('\n');
	var length = arr.length;
	var obj = {};

	for (var i = 0; i < length; i++) {

		var str = arr[i];

		if (str === '')
			continue;

		if (str.substring(0, 2) === '//')
			continue;

		var index = str.indexOf(':');
		if (index === -1)
			continue;

		obj[str.substring(0, index).trim()] = str.substring(index + 1).trim();
	}

	return obj;
};

/*
	@arguments {Object array}
	return {String}
*/
String.prototype.format = function() {
    var formatted = this;
    for (var i = 0; i < arguments.length; i++) {
        var regexp = new RegExp('\\{' + i + '\\}', 'gi');
        formatted = formatted.replace(regexp, arguments[i]);
    }
    return formatted;
};

String.prototype.htmlEncode = function() {
	return this.replace(/\&/g, '&amp;').replace(/\>/g, '&gt;').replace(/\</g, '&lt;').replace(/\"/g, '&quot;');
};

String.prototype.htmlDecode = function() {
	return this.replace(/&gt;/g, '>').replace(/\&lt;/g, '<').replace(/\&quot;/g, '"').replace(/&amp;/g, '&');
};

String.prototype.urlEncode = function() {
	return encodeURIComponent(this);
};

String.prototype.urlDecode = function() {
	return decodeURIComponent(this);
};

/*
	Simple templating :: Hello {name}, your score: {score}, your price: {price | ### ###.##}, date: {date | dd.MM.yyyy}
	@obj {Object}
	return {String}
*/
String.prototype.params = function(obj) {
    var formatted = this.toString();

    if (typeof(obj) === UNDEFINED || obj === null)
		return formatted;

	var reg = /\{[^}\n]*\}/g;
	var match = formatted.match(reg);

	if (match === null)
		return formatted;

	var length = match.length;

	for (var i = 0; i < length; i++) {
		var prop = match[i];

		var isEncode = false;
		var name = prop.substring(1, prop.length - 1).trim();

		var format = '';
		var index = name.indexOf('|');

		if (index !== -1) {
			format = name.substring(index + 1, name.length).trim();
			name = name.substring(0, index).trim();
		}

		if (prop.substring(0, 2) === '{!')
			name = name.substring(1);
		else
			isEncode = true;

		var val;

		if (name.indexOf('.') !== -1) {
			var arr = name.split('.');

			if (arr.length === 2)
				val = obj[arr[0]][arr[1]];
			else if (arr.length === 3)
				val = obj[arr[0]][arr[1]][arr[3]];
			else if (arr.length === 4)
				val = obj[arr[0]][arr[1]][arr[3]][arr[4]];
			else if (arr.length === 5)
				val = obj[arr[0]][arr[1]][arr[3]][arr[4]][arr[5]];

		} else
			val = name.length === 0 ? obj : obj[name];

		if (typeof(val) === FUNCTION)
			val = val(index);

		if (typeof(val) === UNDEFINED)
			return;

		if (format.length > 0) {

			var type = typeof(val);
			if (type === STRING) {
				var max = parseInt(format, 10);
				if (!isNaN(max))
					val = val.max(max + 3, '...');

			} else if (type === NUMBER || util.isDate(val))
				val = val.format(format);
		}

		val = val.toString().dollar();
		formatted = formatted.replace(prop, isEncode ? exports.htmlEncode(val) : val);
	}

    return formatted;
};

/*
	Set max length of string
	@length {Number}
	@chars {String} :: optional, default ...
	return {String}
*/
String.prototype.max = function(length, chars) {
	var str = this.toString();
	chars = chars || '...';
    return str.length > length ? str.substring(0, length - chars.length) + chars : str;
};

String.prototype.maxLength = function(max, chars) {
	console.log('OBSOLETE - String.prototype.maxLength(). Use String.prototype.max()');
	return this.max(max, chars);
};

String.prototype.isJSON = function() {
	var self = this;
	if (self.length <= 1)
		return false;
	var a = self[0];
	var b = self[self.length - 1];
	return (a === '"' && b === '"') || (a === '[' && b === ']') || (a === '{' && b === '}');
};

String.prototype.isURL = function() {
	var str = this.toString();
	if (str.length <= 7)
		return false;
	return regexpUrl.test(str);
};

String.prototype.isEmail = function() {
	var str = this.toString();
	if (str.length <= 4)
		return false;

	if (str[0] === '.' || str[str.length - 1] === '.')
		return false;

	return regexpMail.test(str);
};

/*
	@def {Number} :: optional, default 0
	return {Number}
*/
String.prototype.parseInt = function(def) {
    var num = 0;
    var str = this.toString();

    if (str.substring(0, 1) === '0')
		num = parseInt(str.replace(/\s/g, '').substring(1), 10);
    else
		num = parseInt(str.replace(/\s/g, ''), 10);

    if (isNaN(num))
        return def || 0;

    return num;
};

/*
	@def {Number} :: optional, default 0
	return {Number}
*/
String.prototype.parseFloat = function(def) {
	var num = 0;
    var str = this.toString();

    if (str.substring(0, 1) === '0')
        num = parseFloat(str.replace(/\s/g, '').substring(1).replace(',', '.'));
    else
        num = parseFloat(str.replace(/\s/g, '').replace(',', '.'));

    if (isNaN(num))
        return def || 0;

    return num;
};

String.prototype.toUnicode = function() {
    var result = '';
    var self = this;
    for(var i = 0; i < self.length; ++i){
        if(self.charCodeAt(i) > 126 || self.charCodeAt(i) < 32)
            result += '\\u' + self.charCodeAt(i).hex(4);
        else
            result += self[i];
    }
    return result;
};

String.prototype.fromUnicode = function() {

	var str = this.replace(/\\u([\d\w]{4})/gi, function (match, v) {
		return String.fromCharCode(parseInt(v, 16));
	});

	return unescape(str);
};

String.prototype.sha1 = function() {
	var hash = crypto.createHash('sha1');
	hash.update(this.toString(), ENCODING);
	return hash.digest('hex');
};

String.prototype.md5 = function() {
	var hash = crypto.createHash('md5');
	hash.update(this.toString(), ENCODING);
	return hash.digest('hex');
};

/*
	@key {String}
	@isUnique {Boolean}
	return {String}
*/
String.prototype.encode = function(key, isUnique) {
	var str = '0' + this.toString();
    var data_count = str.length;
    var key_count = key.length;
    var change = str[data_count - 1];
    var random = isUnique || true ? exports.random(120) + 40 : 65;
    var count = data_count + (random % key_count);
    var values = [];
    var index = 0;

    values[0] = String.fromCharCode(random);
    var counter = this.length + key.length;

    for (var i = count - 1; i > 0; i--) {
		index = str.charCodeAt(i % data_count);
		values[i] = String.fromCharCode(index ^ (key.charCodeAt(i % key_count) ^ random));
    }

    var hash = new Buffer(counter + '=' + values.join(''), ENCODING).toString('base64').replace(/\//g, '-').replace(/\+/g, '_');
    index = hash.indexOf('=');
    if (index > 0)
		return hash.substring(0, index);

    return hash;
};

/*
	@key {String}
	return {String}
*/
String.prototype.decode = function(key) {

	var values = this.toString().replace(/\-/g, '/').replace(/\_/g, '+');
	var mod = values.length % 4;

	if (mod > 0) {
		for (var i = 0; i < mod; i++)
			values += '=';
	}

	values = new Buffer(values, 'base64').toString(ENCODING);

	var index = values.indexOf('=');
	if (index === -1)
		return '';

	var counter = parseInt(values.substring(0, index), 10);
	if (isNaN(counter))
		return '';

	values = values.substring(index + 1);

	var count = values.length;
	var random = values.charCodeAt(0);

	var key_count = key.length;
	var data_count = count - (random % key_count);
	var decrypt_data = [];

	for (var i = data_count - 1; i > 0; i--) {
		index = values.charCodeAt(i) ^ (random ^ key.charCodeAt(i % key_count));
		decrypt_data[i] = String.fromCharCode(index);
	}

	var val = decrypt_data.join('');

	if (counter !== val.length + key.length)
		return '';

	return val;
};

/*
	Convert value from base64 and save to file
	@filename {String}
	@callback {Function} :: optional
	return {String}
*/
String.prototype.base64ToFile = function(filename, callback) {
	var self = this.toString();

	var index = self.indexOf(',');
	if (index === -1)
		index = 0;
	else
		index++;

	if (callback)
		fs.writeFile(filename, self.substring(index), 'base64', callback);
	else
		fs.writeFile(filename, self.substring(index), 'base64', exports.noop);

	return this;
};

/*
	Get content type from base64
	return {String}
*/
String.prototype.base64ContentType = function() {
	var self = this.toString();

	var index = self.indexOf(';');
	if (index === -1)
		return '';

	return self.substring(5, index);
};

String.prototype.removeDiacritics = function() {
	return exports.removeDiacritics(this);
};

/*
	Indent
	@max {Number}
	@c {String} : optional, default SPACE
	return {String}
*/
String.prototype.indent = function(max, c) {
	var self = this.toString();
	return new Array(max + 1).join(c || ' ') + self;
};

/*
	isNumber?
	@isDecimal {Boolean} :: optional, default false
	return {Boolean}
*/
String.prototype.isNumber = function(isDecimal) {

	var self = this.toString();

	if (self.length === 0)
		return false;

	isDecimal = isDecimal || false;

	for (var i = 0; i < self.length; i++) {
		var ascii = self.charCodeAt(i);

		if (isDecimal) {
			if (ascii === 44 || ascii === 46) {
				isDecimal = false;
				continue;
			}
		}

		if (ascii < 48 || ascii > 57)
			return false;
	}

	return true;
};

/*
	@max {Number}
	@c {String} :: optional
	return {String}
*/
String.prototype.padLeft = function(max, c) {
	var self = this.toString();
	return new Array(Math.max(0, max - self.length + 1)).join(c || ' ') + self;
};

/*
	@max {Number}
	@c {String} :: optional
	return {String}
*/
String.prototype.padRight = function(max, c) {
	var self = this.toString();
	return self + new Array(Math.max(0, max - self.length + 1)).join(c || ' ');
};

/*
	index {Number}
	value {String}
	return {String}
*/
String.prototype.insert = function(index, value) {
	var str = this.toString();
	var a = str.substring(0, index);
	var b = value.toString() + str.substring(index);
	return a + b;
};

/*
	Prepare string for replacing double dollar
*/
String.prototype.dollar = function() {
	var str = this.toString();
	var index = str.indexOf('$', 0);

	while (index !== -1) {
		if (str[index + 1] === '$')
			str = str.insert(index, '$');
		index = str.indexOf('$', index + 2);
	}
	return str.toString();
};

/*
	Create link
	@max {Number} :: optional default 60 chars
	return {String}
*/
String.prototype.link = function(max) {
	max = max || 60;

	var self = this.toString().trim().removeDiacritics().toLowerCase();
	var builder = [];

	for (var i = 0; i < self.length; i++) {
		var c = self[i];
		var code = self.charCodeAt(i);

		if (builder.length >= max)
			break;

		if (code > 31 && code < 48) {

			if (builder[builder.length - 1] === '-')
				continue;

			builder.push('-');
			continue;
		}

		if (code > 47 && code < 58) {
			builder.push(c);
			continue;
		}

		if (code > 94 && code < 123) {
			builder.push(c);
			continue;
		}

	}

	return builder.join('');
};

String.prototype.pluralize = function(zero, one, few, other) {
	var str = this.toString();
	return str.parseInt().pluralize(zero, one, few, other);
};

String.prototype.isBoolean = function() {
	var self = this.toString().toLowerCase();
	return (self === 'true' || self === 'false') ? true : false;
};

/*
	@decimals {Number}
	return {Number}
*/
Number.prototype.floor = function(decimals) {
	return Math.floor(this * Math.pow(10, decimals)) / Math.pow(10, decimals);
};

/*
	@max {Number}
	@c {String} :: optional
	return {String}
*/
Number.prototype.padLeft = function(max, c) {
	return this.toString().padLeft(max, c || '0');
};

/*
	@max {Number}
	@c {String} :: optional
	return {String}
*/
Number.prototype.padRight = function(max, c) {
	return this.toString().padRight(max, c || '0');
};

/*
	Format number :: 10000 = 10 000
	@format {Number or String} :: number is decimal and string is specified format, example: ## ###.##
	return {String}
*/
Number.prototype.format = function(format) {

	var index = 0;
	var num = this.toString();
	var beg = 0;
	var end = 0;
	var max = 0;
	var output = '';

	if (typeof(format) === STRING) {

		var d = false;

		for (var i = 0; i < format.length; i++) {
			var c = format[i];
			if (c === '#') {
				if (d)
					end++;
				else
					beg++;
			}

			if (c === '.')
				d = true;
		}

		var strBeg = num;
		var strEnd = '';

		index = num.indexOf('.');

		if (index !== -1) {
			strBeg = num.substring(0, index);
			strEnd = num.substring(index + 1);
		}

		if (strBeg.length > beg) {
			max = strBeg.length - beg;
			var tmp = '';
			for (var i = 0; i < max; i++)
				tmp += '#';

			format = tmp + format;
		}

		if (strBeg.length < beg)
			strBeg = strBeg.padLeft(beg, ' ');

		if (strEnd.length < end)
			strEnd = strEnd.padRight(end, '0');

		if (strEnd.length > end)
			strEnd = strEnd.substring(0, end);

		d = false;
		index = 0;

		var skip = true;

		for (var i = 0; i < format.length; i++) {

			var c = format[i];

			if (c !== '#') {

				if (skip)
					continue;

				if (c === '.') {
					d = true;
					index = 0;
				}

				output += c;
				continue;
			}

			var value = d ? strEnd[index] : strBeg[index];

			if (skip)
				skip = [',', ' '].indexOf(value) !== -1;

			if (!skip)
				output += value;

			index++;
		}

		return output;
	}

	output = '### ### ###';
	beg = num.indexOf('.');
	max = format || 0;

	if (max === 0 && num != -1)
		max = num.length - (beg + 1);

	if (max > 0) {
		output += '.';
		for (var i = 0; i < max; i++)
			output += '#';
	}

	return this.format(output);
};

/*
	Pluralize number
	zero {String}
	one {String}
	few {String}
	other {String}
	return {String}
*/
Number.prototype.pluralize = function(zero, one, few, other) {

	var num = this;
	var value = '';

	if (num === 0)
		value = zero || '';
	else if (num === 1)
		value = one || '';
	else if (num > 1 && num < 5)
		value = few || '';
	else
		value = other;

	var beg = value.indexOf('#');
	var end = value.lastIndexOf('#');

	if (beg === -1)
		return value;

	var format = value.substring(beg, end + 1);
	return num.format(format) + value.replace(format, '');
};

/*
	@length {Number}
	return {String}
*/
Number.prototype.hex = function(length) {
    var str = this.toString(16).toUpperCase();
    while(str.length < length)
        str = '0' + str;
    return str;
};

Number.prototype.condition = function(ifTrue, ifFalse) {
	return (this % 2 === 0 ? ifTrue : ifFalse) || '';
};

Boolean.prototype.condition = function(ifTrue, ifFalse) {
	return (this ? ifTrue : ifFalse) || '';
};

/*
    @count {Number}
	return {Array}
*/
Array.prototype.take = function(count) {
	var arr = [];
	var self = this;
	for (var i = 0; i < self.length; i++) {
		arr.push(self[i]);
		if (arr.length >= count)
			return arr;
	}
	return arr;
};

/*
    @count {Number}
    return {Array}
*/
Array.prototype.skip = function(count) {
	var arr = [];
	var self = this;
	for (var i = 0; i < self.length; i++) {
		if (i >= count)
			arr.push(self[i]);
	}
	return arr;
};

/*
	@cb {Function} :: return true / false
	return {Array}
*/
Array.prototype.where = function(cb) {

	var self = this;
	var selected = [];

	for (var i = 0; i < self.length; i++) {
		if (cb.call(self, self[i], i))
			selected.push(self[i]);
	}

	return selected;
};

/*
	@cb {Function} :: return true if is finded
	return {Array item}
*/
Array.prototype.find = function(cb) {
	var self = this;
	for (var i = 0; i < self.length; i++) {
		if (cb.call(self, self[i], i))
			return self[i];
	}
	return null;
};

/*
	@cb {Function} :: return true if is removed
	return {Array}
*/
Array.prototype.remove = function(cb) {
	var self = this;
	var arr = [];
	for (var i = 0; i < self.length; i++) {
		if (!cb.call(self, self[i], i))
			arr.push(self[i]);
	}
	return arr;
};

/*
	Random return item from array
	Return {Object}
*/
Array.prototype.random = function() {
	var self = this;
	return self[exports.random(self.length - 1)];
};

/*
	Randomize array
	Return {Array}
*/
Array.prototype.randomize = function() {

	var self = this;
	var random = (Math.floor(Math.random() * 100000000) * 10).toString();
	var index = 0;
	var old = 0;

	self.sort(function(a, b) {

		var c = random[index++];

		if (typeof(c) === UNDEFINED) {
			c = random[0];
			index = 0;
		}

		if (old > c) {
			old = c;
			return -1;
		}

		if (old === c) {
			old = c;
			return 0;
		}

		old = c;
		return 1;
	});

	return self;
};

/*
	Async class
*/
function AsyncTask(owner, name, fn, cb, waiting) {

	this.handlers = {
		oncomplete: this.complete.bind(this)
	};

	this.isRunning = 0;
	this.owner = owner;
	this.name = name;
	this.fn = fn;
	this.cb = cb;
	this.waiting = waiting;
	this.interval = null;
	this.isCanceled = false;
}

AsyncTask.prototype.run = function() {
	var self = this;
	try
	{
		self.isRunning = 1;
		self.owner.tasksWaiting[self.name] = true;
		self.owner.emit('begin', self.name);

		var timeout = self.owner.tasksTimeout[self.name];
		if (timeout > 0)
			self.interval = setTimeout(self.timeout.bind(self), timeout);

		self.fn(self.handlers.oncomplete);
	} catch (ex) {
		self.owner.emit('error', self.name, ex);
		self.complete();
	}
	return self;
};

AsyncTask.prototype.timeout = function(timeout) {
	var self = this;

	if (timeout > 0) {
		clearTimeout(self.interval);
		setTimeout(self.timeout.bind(self), timeout);
		return self;
	}

	if (timeout <= 0) {
		clearTimeout(self.interval);
		setTimeout(self.timeout.bind(self), timeout);
		return self;
	}

	self.cancel(true);
	return self;
};

AsyncTask.prototype.cancel = function(isTimeout) {
	var self = this;

	self.isCanceled = true;

	if (isTimeout)
		self.owner.emit('timeout', self.name);
	else
		self.owner.emit('cancel', self.name);

	self.fn = null;
	self.cb = null;
};

AsyncTask.prototype.complete = function() {

	var item = this;
	var self = item.owner;

	item.isRunning = 2;

	delete self.tasksPending[item.name];
	delete self.tasksWaiting[item.name];

	if (!item.isCanceled) {
		try
		{
			self.emit('end', item.name);

			if (item.cb)
				item.cb();

		} catch (ex) {
			self.emit('error', ex, item.name);
		}
	}

	self.reload();
	self.refresh();

	return self;
};

function Async(owner) {

	this._max = 0;
	this._count = 0;
	this._isRunning = false;

	this.owner = owner;
	this.onComplete = [];

	this.tasksPending = {};
	this.tasksWaiting = {};
	this.tasksAll = [];
	this.tasksTimeout = {};
}

Async.prototype = {
	get count() {
		return this._count;
	},

	get percentage() {
		var self = this;
		return 100 - Math.floor((self._count * 100) / self._max);
	}
}

Async.prototype.__proto__ = new events.EventEmitter();

Async.prototype.reload = function() {
	var self = this;
	self.tasksAll = Object.keys(self.tasksPending);
	self.emit('percentage', self.percentage);
	return self;
};

Async.prototype.cancel = function(name) {

	var self = this;

	if (typeof(name) === UNDEFINED) {

		for (var i = 0; i < self._count; i++)
			self.cancel(tasksAll[i]);

		return true;
	}

	var task = self.tasksPending[name];

	if (!task)
		return false;

	delete self.tasksPending[name];
	delete self.tasksWaiting[name];

	task.cancel();
	self.reload();
	self.refresh();

	return true;
};

Async.prototype.await = function(name, fn, cb) {

	var self = this;

	if (typeof(name) === FUNCTION) {
		cb = fn;
		fn = name;
		name = exports.GUID(6);
	}

	if (typeof(self.tasksPending[name]) !== UNDEFINED)
		return false;

	self.tasksPending[name] = new AsyncTask(self, name, fn, cb, null);
	self._max++;
	self.reload();
	self.refresh();

	return true;
};

Async.prototype.wait = function(name, waitingFor, fn, cb) {

	var self = this;

	if (typeof(waitingFor) === FUNCTION) {
		cb = fn;
		fn = waitingFor;
		waitingFor = name;
		name = exports.GUID(6);
	}

	if (typeof(self.tasksPending[name]) !== UNDEFINED)
		return false;

	self.tasksPending[name] = new AsyncTask(self, name, fn, cb, waitingFor);
	self._max++;
	self.reload();
	self.refresh();

	return true;

};

Async.prototype.complete = function(fn) {
	return this.run(fn);
};

Async.prototype.run = function(fn) {
	var self = this;
	self._isRunning = true;

	if (fn)
		self.onComplete.push(fn);

	self.refresh();
	return self;
};

Async.prototype.isRunning = function(name) {

	var self = this;

	if (!name)
		return self._isRunning;

	var task = self.tasksPending[name];
	if (!task)
		return false;

	return task.isRunning === 1;
};

Async.prototype.isWaiting = function(name) {
	var self = this;

	var task = self.tasksPending[name];
	if (!task)
		return false;

	return task.isRunning === 0;
};

Async.prototype.isPending = function(name) {
	var self = this;
	var task = self.tasksPending[name];
	if (!task)
		return false;
	return true;
};

Async.prototype.timeout = function(name, timeout) {

	var self = this;

	if (timeout <= 0 || typeof(timeout) === UNDEFINED) {
		delete self.tasksTimeout[name];
		return self;
	}

	self.tasksTimeout[name] = timeout;
	return self;
};

Async.prototype.refresh = function(name) {

	var self = this;

	if (!self._isRunning)
		return self;

	self._count = self.tasksAll.length;

	for (var i = 0; i < self._count; i++) {

		var task = self.tasksPending[self.tasksAll[i]];

		if (task.isRunning !== 0)
			continue;

		if (task.waiting !== null && typeof(self.tasksWaiting[task.waiting]) !== UNDEFINED)
			continue;

		task.run();
	}

	if (self._count === 0) {
		self._isRunning = false;
		self.emit('complete');
		self.emit('percentage', 100);
		self._max = 0;
		var length = self.onComplete.length;
		for (var i = 0; i < length; i++) {
			try
			{
				self.onComplete[i]();
			} catch (ex) {
				self.emit('error', ex);
			}
		}
		self.onComplete = [];
	}

	return self;
};

exports.Async = Async;
exports.async = Async;
